@@ -51,12 +51,11 @@ static void start_as_daemon (void);
 
 Display *current_display;  // The current display
 
-#ifndef GUILE_FLAG
 extern char rc_file[512];
-#else
+#ifdef GUILE_FLAG
 extern char rc_guile_file[512];
 #endif
-
+extern int poll_rc;
 
 #define SLEEP_TIME 100
 
@@ -202,7 +201,7 @@ static void
 adjust_display (XAnyEvent * xany)
 {
   size_t envstr_size = strlen(DisplayString(xany->display)) + 8 + 1;
-  char* envstr = malloc (envstr_size);
+  char* envstr = malloc ( (envstr_size + 2) * sizeof (char) );
   XWindowAttributes attr;
   char* ptr;
   char buf[16];
@@ -232,54 +231,52 @@ event_loop (Display * d)
   XEvent e;
   int i;
   struct stat rc_file_info;
-#ifndef GUILE_FLAG
-  time_t rc_file_changed;
-#else
-  time_t rc_guile_file_changed;
+  time_t rc_file_changed = 0;
+#ifdef GUILE_FLAG
+  time_t rc_guile_file_changed = 0;
+  struct stat rc_guile_file_info;
 #endif
 
 
-
   XSetErrorHandler ((XErrorHandler) null_X_error);
 
-#ifndef GUILE_FLAG
-  stat(rc_file, &rc_file_info);
-  rc_file_changed = rc_file_info.st_mtime;
-#else
-  stat (rc_guile_file, &rc_file_info);
-  rc_guile_file_changed = rc_file_info.st_mtime;
+  if (poll_rc)
+    {
+      stat(rc_file, &rc_file_info);
+      rc_file_changed = rc_file_info.st_mtime;
+#ifdef GUILE_FLAG
+      stat (rc_guile_file, &rc_guile_file_info);
+      rc_guile_file_changed = rc_guile_file_info.st_mtime;
 #endif
-
+    }
 
   while (True)
     {
-      while(!XPending(d))
+      while(poll_rc && !XPending(d))
 	{
-#ifndef GUILE_FLAG
 	  // if the rc file has been modified, reload it
 	  stat (rc_file, &rc_file_info);
-	  if (rc_file_info.st_mtime != rc_file_changed)
+#ifdef GUILE_FLAG
+	  // if the rc guile file has been modified, reload it
+	  stat (rc_guile_file, &rc_guile_file_info);
+#endif
+
+	  if (rc_file_info.st_mtime != rc_file_changed
+#ifdef GUILE_FLAG
+	      || rc_guile_file_info.st_mtime != rc_guile_file_changed
+#endif
+	      )
 	    {
 	      reload_rc_file ();
 	      if (verbose)
 		{
-		  printf ("The rc file has been modified, reload it\n");
+		  printf ("The configuration file has been modified, reload it\n");
 		}
 	      rc_file_changed = rc_file_info.st_mtime;
-	    }
-#else
-	  // if the rc guile file has been modified, reload it
-	  stat (rc_guile_file, &rc_file_info);
-	  if (rc_file_info.st_mtime != rc_guile_file_changed)
-	    {
-	      reload_rc_file ();
-	      if (verbose)
-		{
-		  printf ("The rc guile file has been modified, reload it\n");
-		}
-	      rc_guile_file_changed = rc_file_info.st_mtime;
-	    }
+#ifdef GUILE_FLAG
+	      rc_guile_file_changed = rc_guile_file_info.st_mtime;
 #endif
+	    }
 
 	  usleep(SLEEP_TIME*1000);
 	}
